%% $Id: pstricks-xetex.def 276 2021-09-27 11:28:22Z herbert $
%%
% This work may be distributed and/or modified under the
% conditions of the LaTeX Project Public License, either version 1.3
% of this license or (at your option) any later version.
% The latest version of this license is in
%   http://www.latex-project.org/lppl.txt
% and version 1.3 or later is part of all distributions of LaTeX
% version 2003/12/01 or later.
%
% The xdvipdfmx driver does not support all node macros. In the following
% there are alternatives defined:
%
%% fileversion and filedate see main file pstricks.tex
%%  \ncput -> \NCput
%
\def\NCput{\pst@object{NCput}}
\def\NCput@i#1{%
  \begingroup
  \use@par
  \pst@getcoor{@@A}\pst@tempA% defined by \pcline and \ncline
  \pst@getcoor{@@B}\pst@tempB
  \if@star\def\next{\rput*}\else\def\next{\rput}\fi
  \next(!
  	  \pst@tempA /yA ED /xA ED
  	  \pst@tempB /yB ED /xB ED
  	  xA xB add 2 div
  	  yA yB add 2 div \tx@UserCoor){#1}
  \endgroup
  \ignorespaces
}
%%
%% END: pstricks-xetex.def
