%%
%% Package `Vaucanson-G'  version 0.4
%%
%% This is file `vcsn-color-names.def'.
%%
%% IMPORTANT NOTICE:
%%
%% Copyright (C) 2002-2008 Sylvain Lombardy and Jacques Sakarovitch
%%
%% This package may be distributed under the terms of the LaTeX Project
%% Public License, as described in lppl.txt in the base LaTeX distribution.
%% Either version 1.0 or, at your option, any later version.
%%
%% DESCRIPTION:
%%
%%   `vcsn-color-names.def' provides usual color definitions for Vaucanson-G
%%  
\xdefinecolor{GreenYellow}   {cmyk}{0.15,0,0.69,0}
\xdefinecolor{Yellow}        {cmyk}{0,0,1,0}
\xdefinecolor{Goldenrod}     {cmyk}{0,0.10,0.84,0}
\xdefinecolor{Dandelion}     {cmyk}{0,0.29,0.84,0}
\xdefinecolor{Apricot}       {cmyk}{0,0.32,0.52,0}
\xdefinecolor{Peach}         {cmyk}{0,0.50,0.70,0}
\xdefinecolor{Melon}         {cmyk}{0,0.46,0.50,0}
\xdefinecolor{YellowOrange}  {cmyk}{0,0.42,1,0}
\xdefinecolor{Orange}        {cmyk}{0,0.61,0.87,0}
\xdefinecolor{BurntOrange}   {cmyk}{0,0.51,1,0}
\xdefinecolor{Bittersweet}   {cmyk}{0,0.75,1,0.24}
\xdefinecolor{RedOrange}     {cmyk}{0,0.77,0.87,0}
\xdefinecolor{Mahogany}      {cmyk}{0,0.85,0.87,0.35}
\xdefinecolor{Maroon}        {cmyk}{0,0.87,0.68,0.32}
\xdefinecolor{BrickRed}      {cmyk}{0,0.89,0.94,0.28}
\xdefinecolor{Red}           {cmyk}{0,1,1,0}
\xdefinecolor{OrangeRed}     {cmyk}{0,1,0.50,0}
\xdefinecolor{RubineRed}     {cmyk}{0,1,0.13,0}
\xdefinecolor{WildStrawberry}{cmyk}{0,0.96,0.39,0}
\xdefinecolor{Salmon}        {cmyk}{0,0.53,0.38,0}
\xdefinecolor{CarnationPink} {cmyk}{0,0.63,0,0}
\xdefinecolor{Magenta}       {cmyk}{0,1,0,0}
\xdefinecolor{VioletRed}     {cmyk}{0,0.81,0,0}
\xdefinecolor{Rhodamine}     {cmyk}{0,0.82,0,0}
\xdefinecolor{Mulberry}      {cmyk}{0.34,0.90,0,0.02}
\xdefinecolor{RedViolet}     {cmyk}{0.07,0.90,0,0.34}
\xdefinecolor{Fuchsia}       {cmyk}{0.47,0.91,0,0.08}
\xdefinecolor{Lavender}      {cmyk}{0,0.48,0,0}
\xdefinecolor{Thistle}       {cmyk}{0.12,0.59,0,0}
\xdefinecolor{Orchid}        {cmyk}{0.32,0.64,0,0}
\xdefinecolor{DarkOrchid}    {cmyk}{0.40,0.80,0.20,0}
\xdefinecolor{Purple}        {cmyk}{0.45,0.86,0,0}
\xdefinecolor{Plum}          {cmyk}{0.50,1,0,0}
\xdefinecolor{Violet}        {cmyk}{0.79,0.88,0,0}
\xdefinecolor{RoyalPurple}   {cmyk}{0.75,0.90,0,0}
\xdefinecolor{BlueViolet}    {cmyk}{0.86,0.91,0,0.04}
\xdefinecolor{Periwinkle}    {cmyk}{0.57,0.55,0,0}
\xdefinecolor{CadetBlue}     {cmyk}{0.62,0.57,0.23,0}
\xdefinecolor{CornflowerBlue}{cmyk}{0.65,0.13,0,0}
\xdefinecolor{MidnightBlue}  {cmyk}{0.98,0.13,0,0.43}
\xdefinecolor{NavyBlue}      {cmyk}{0.94,0.54,0,0}
\xdefinecolor{RoyalBlue}     {cmyk}{1,0.50,0,0}
\xdefinecolor{Blue}          {cmyk}{1,1,0,0}
\xdefinecolor{Cerulean}      {cmyk}{0.94,0.11,0,0}
\xdefinecolor{Cyan}          {cmyk}{1,0,0,0}
\xdefinecolor{ProcessBlue}   {cmyk}{0.96,0,0,0}
\xdefinecolor{SkyBlue}       {cmyk}{0.62,0,0.12,0}
\xdefinecolor{Turquoise}     {cmyk}{0.85,0,0.20,0}
\xdefinecolor{TealBlue}      {cmyk}{0.86,0,0.34,0.02}
\xdefinecolor{Aquamarine}    {cmyk}{0.82,0,0.30,0}
\xdefinecolor{BlueGreen}     {cmyk}{0.85,0,0.33,0}
\xdefinecolor{Emerald}       {cmyk}{1,0,0.50,0}
\xdefinecolor{JungleGreen}   {cmyk}{0.99,0,0.52,0}
\xdefinecolor{SeaGreen}      {cmyk}{0.69,0,0.50,0}
\xdefinecolor{Green}         {cmyk}{1,0,1,0}
\xdefinecolor{ForestGreen}   {cmyk}{0.91,0,0.88,0.12}
\xdefinecolor{PineGreen}     {cmyk}{0.92,0,0.59,0.25}
\xdefinecolor{LimeGreen}     {cmyk}{0.50,0,1,0}
\xdefinecolor{YellowGreen}   {cmyk}{0.44,0,0.74,0}
\xdefinecolor{SpringGreen}   {cmyk}{0.26,0,0.76,0}
\xdefinecolor{OliveGreen}    {cmyk}{0.64,0,0.95,0.40}
\xdefinecolor{RawSienna}     {cmyk}{0,0.72,1,0.45}
\xdefinecolor{Sepia}         {cmyk}{0,0.83,1,0.70}
\xdefinecolor{Brown}         {cmyk}{0,0.81,1,0.60}
\xdefinecolor{Tan}           {cmyk}{0.14,0.42,0.56,0}
\xdefinecolor{Gray}          {cmyk}{0,0,0,0.50}
\xdefinecolor{Black}         {cmyk}{0,0,0,1}
\xdefinecolor{White}         {cmyk}{0,0,0,0}
\endinput
%%
